package net.shadowfacts.phycon.network.block.extractor

import alexiil.mc.lib.attributes.SearchOptions
import alexiil.mc.lib.attributes.item.FixedItemInv
import alexiil.mc.lib.attributes.item.GroupedItemInv
import alexiil.mc.lib.attributes.item.ItemAttributes
import net.minecraft.util.math.Direction
import net.shadowfacts.phycon.api.packet.Packet
import net.shadowfacts.phycon.api.util.IPAddress
import net.shadowfacts.phycon.init.PhyBlockEntities
import net.shadowfacts.phycon.network.DeviceBlockEntity
import net.shadowfacts.phycon.network.packet.CapacityPacket
import net.shadowfacts.phycon.network.packet.CheckCapacityPacket
import net.shadowfacts.phycon.network.packet.ItemStackPacket

/**
 * @author shadowfacts
 */
class ExtractorBlockEntity: DeviceBlockEntity(PhyBlockEntities.EXTRACTOR) {

	private val facing: Direction
		get() = cachedState[ExtractorBlock.FACING]

	private var inventory: GroupedItemInv? = null
	private var shouldExtract = false

	fun updateInventory() {
		val offsetPos = pos.offset(facing)
		val option = SearchOptions.inDirection(facing)
		inventory = ItemAttributes.GROUPED_INV.getFirstOrNull(world, offsetPos, option)
	}

	private fun getInventory(): GroupedItemInv? {
		if (inventory == null) updateInventory()
		return inventory
	}

	override fun handle(packet: Packet) {
		if (packet is CapacityPacket && shouldExtract) {
			getInventory()?.also { inv ->
				shouldExtract = false
				val extracted = inv.extract(packet.stack, packet.capacity)
				sendPacket(ItemStackPacket(extracted, ipAddress, packet.stackReceiver.ipAddress))
			}
		}
	}

	override fun tick() {
		super.tick()

		if (!world!!.isClient && counter % 40 == 0L) {
			getInventory()?.also {
				val stack = it.storedStacks.firstOrNull() ?: return
				shouldExtract = true
				sendPacket(CheckCapacityPacket(stack, ipAddress, IPAddress.BROADCAST))
			}
		}
	}
}